package com.wieseke.cptk.app;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarContributionItem;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.ContributionItemFactory;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;

/**
 * An action bar advisor is responsible for creating, adding, and disposing of
 * the actions added to a workbench window. Each window will be populated with
 * new actions.
 */
public class CPTKActionBarAdvisor extends ActionBarAdvisor {

	private Map<ActionFactory, IWorkbenchAction> actions = new HashMap<ActionFactory, IWorkbenchAction>();

	private IWorkbenchWindow window;

	// Actions - important to allocate these only in makeActions, and then use
	// them
	// in the fill methods. This ensures that the actions aren't recreated
	// when fillActionBars is called with FILL_PROXY.

	public CPTKActionBarAdvisor(IActionBarConfigurer configurer) {
		super(configurer);
	}

	protected void makeActions(final IWorkbenchWindow window) {
		// Creates the actions and registers them.
		// Registering is needed to ensure that key bindings work.
		// The corresponding commands keybindings are defined in the plugin.xml
		// file.
		// Registering also provides automatic disposal of the actions when
		// the window is closed.

		this.window = window;
		ActionFactory actionsToCreate[] = new ActionFactory[] {
				ActionFactory.NEW_WIZARD_DROP_DOWN, ActionFactory.NEW,
				ActionFactory.CLOSE, ActionFactory.CLOSE_ALL,
				ActionFactory.SAVE, ActionFactory.SAVE_AS,
				ActionFactory.SAVE_ALL, ActionFactory.REVERT,
				ActionFactory.MOVE, ActionFactory.RENAME,
				ActionFactory.REFRESH, ActionFactory.PRINT,
				ActionFactory.IMPORT, ActionFactory.EXPORT,
				ActionFactory.PROPERTIES, ActionFactory.QUIT,
				ActionFactory.UNDO, ActionFactory.REDO, ActionFactory.CUT,
				ActionFactory.COPY, ActionFactory.PASTE, ActionFactory.DELETE,
				ActionFactory.SELECT_ALL, ActionFactory.FIND,
				ActionFactory.PREFERENCES, ActionFactory.ABOUT };

		for (ActionFactory factory : actionsToCreate) {
			IWorkbenchAction action = factory.create(window);
			actions.put(factory, action);
			register(action);
		}

		// removeUnusedActionSets();

	}

	// private void removeUnusedActionSets() {
	// ActionSetRegistry reg = WorkbenchPlugin.getDefault()
	// .getActionSetRegistry();
	// for (IActionSetDescriptor actionSet : reg.getActionSets()) {
	// if (actionSet.getId().equals(
	// "org.eclipse.ui.edit.text.actionSet.navigation")
	// || actionSet
	// .getId()
	// .equals(
	// "org.eclipse.ui.edit.text.actionSet.convertLineDelimitersTo")
	// || actionSet
	// .getId()
	// .equals(
	// "org.eclipse.ui.edit.text.actionSet.annotationNavigation")) {
	// IExtension ext = actionSet.getConfigurationElement()
	// .getDeclaringExtension();
	// reg.removeExtension(ext, new Object[] { actionSet });
	// }
	// }
	// }

	protected void fillCoolBar(ICoolBarManager cbManager) {
		cbManager.add(new GroupMarker("group.file"));
		{
			IToolBarManager toolBar = new ToolBarManager(cbManager.getStyle());
			toolBar.add(new Separator(IWorkbenchActionConstants.NEW_GROUP));
			toolBar.add(actions.get(ActionFactory.NEW_WIZARD_DROP_DOWN));
			toolBar.add(new GroupMarker(IWorkbenchActionConstants.OPEN_EXT));
			toolBar.add(new GroupMarker(IWorkbenchActionConstants.SAVE_GROUP));
			toolBar.add(actions.get(ActionFactory.SAVE));

			toolBar.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

			// Add to the cool bar manager
			cbManager.add(new ToolBarContributionItem(toolBar,
					IWorkbenchActionConstants.TOOLBAR_FILE));
		}

		cbManager.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));

		cbManager.add(new GroupMarker(IWorkbenchActionConstants.GROUP_EDITOR));
	}

	protected void fillMenuBar(IMenuManager menubar) {
		menubar.add(createFileMenu());
		menubar.add(createEditMenu());
		menubar.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		menubar.add(createWindowMenu());
		menubar.add(createHelpMenu());
	}

	/**
	 * Creates and returns the File menu.
	 */
	private MenuManager createFileMenu() {
		MenuManager fileMenu = new MenuManager("&File",
				IWorkbenchActionConstants.M_FILE);

		fileMenu.add(new GroupMarker(IWorkbenchActionConstants.FILE_START));
		MenuManager newMenu = new MenuManager("New", ActionFactory.NEW.getId());
		newMenu.add(new Separator(ActionFactory.NEW.getId()));
		// newMenu.add(org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard)
		newMenu
				.add(ContributionItemFactory.NEW_WIZARD_SHORTLIST
						.create(window));
		newMenu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		fileMenu.add(newMenu);
		fileMenu.add(new GroupMarker(IWorkbenchActionConstants.NEW_EXT));
		fileMenu.add(new GroupMarker(IWorkbenchActionConstants.OPEN_EXT));
		fileMenu.add(new Separator());

		fileMenu.add(actions.get(ActionFactory.CLOSE));
		fileMenu.add(actions.get(ActionFactory.CLOSE_ALL));
		fileMenu.add(new GroupMarker(IWorkbenchActionConstants.CLOSE_EXT));
		fileMenu.add(new Separator());

		fileMenu.add(actions.get(ActionFactory.SAVE));
		fileMenu.add(actions.get(ActionFactory.SAVE_AS));
		fileMenu.add(actions.get(ActionFactory.SAVE_ALL));
		fileMenu.add(actions.get(ActionFactory.REVERT));
		fileMenu.add(new GroupMarker(IWorkbenchActionConstants.SAVE_EXT));
		fileMenu.add(new Separator());

		fileMenu.add(actions.get(ActionFactory.MOVE));
		fileMenu.add(actions.get(ActionFactory.RENAME));
		fileMenu.add(actions.get(ActionFactory.REFRESH));
		fileMenu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		fileMenu.add(new Separator());

		fileMenu.add(actions.get(ActionFactory.PRINT));
		fileMenu.add(new GroupMarker(IWorkbenchActionConstants.PRINT_EXT));
		fileMenu.add(new Separator());

		fileMenu.add(actions.get(ActionFactory.IMPORT));
		fileMenu.add(actions.get(ActionFactory.EXPORT));
		fileMenu.add(new GroupMarker(IWorkbenchActionConstants.IMPORT_EXT));
		fileMenu.add(new Separator());

		fileMenu.add(actions.get(ActionFactory.PROPERTIES));

		fileMenu.add(ContributionItemFactory.REOPEN_EDITORS.create(window));
		fileMenu.add(new GroupMarker(IWorkbenchActionConstants.MRU));
		fileMenu.add(new Separator());

		fileMenu.add(actions.get(ActionFactory.QUIT));

		fileMenu.add(new GroupMarker(IWorkbenchActionConstants.FILE_END));

		return fileMenu;
	}

	/**
	 * Creates and returns the Edit menu.
	 */
	private MenuManager createEditMenu() {
		MenuManager editMenu = new MenuManager("&Edit",
				IWorkbenchActionConstants.M_EDIT);

		editMenu.add(new GroupMarker(IWorkbenchActionConstants.EDIT_START));

		editMenu.add(actions.get(ActionFactory.UNDO));
		editMenu.add(actions.get(ActionFactory.REDO));
		editMenu.add(new GroupMarker(IWorkbenchActionConstants.UNDO_EXT));
		editMenu.add(new Separator());

		editMenu.add(actions.get(ActionFactory.CUT));
		editMenu.add(actions.get(ActionFactory.COPY));
		editMenu.add(actions.get(ActionFactory.PASTE));
		editMenu.add(new GroupMarker(IWorkbenchActionConstants.CUT_EXT));
		editMenu.add(new Separator());

		editMenu.add(actions.get(ActionFactory.DELETE));
		editMenu.add(actions.get(ActionFactory.SELECT_ALL));
		editMenu.add(new Separator());

		editMenu.add(actions.get(ActionFactory.FIND));
		editMenu.add(new GroupMarker(IWorkbenchActionConstants.FIND_EXT));
		editMenu.add(new Separator());

		editMenu.add(new GroupMarker(IWorkbenchActionConstants.ADD_EXT));
		editMenu.add(new GroupMarker(IWorkbenchActionConstants.EDIT_END));
		editMenu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		editMenu.add(new Separator("preferences")); //$NON-NLS-1$
		editMenu.add(actions.get(ActionFactory.PREFERENCES));
		return editMenu;
	}

	/**
	 * Creates and returns the Help menu.
	 */
	private MenuManager createWindowMenu() {
		MenuManager windowMenu = new MenuManager("&Window",
				IWorkbenchActionConstants.M_WINDOW);
		windowMenu.add(new GroupMarker(IWorkbenchActionConstants.VIEW_EXT));
		MenuManager viewMenu = new MenuManager("Show View");
		IContributionItem viewList = ContributionItemFactory.VIEWS_SHORTLIST
				.create(window);
		viewMenu.add(viewList);
		windowMenu.add(viewMenu);
		windowMenu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		return windowMenu;
	}

	/**
	 * Creates and returns the Help menu.
	 */
	private MenuManager createHelpMenu() {
		MenuManager helpMenu = new MenuManager("&Help",
				IWorkbenchActionConstants.M_HELP);
		helpMenu.add(new GroupMarker(IWorkbenchActionConstants.HELP_START));
		helpMenu.add(new GroupMarker(IWorkbenchActionConstants.HELP_END));
		helpMenu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		helpMenu.add(new Separator("about")); //$NON-NLS-1$
		helpMenu.add(getAction(ActionFactory.ABOUT.getId()));
		return helpMenu;
	}
}
