/**
 * 
 */
package com.wieseke.cptk.common.dao;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.wieseke.cptk.common.api.ICophylogeny;
import com.wieseke.cptk.common.api.IHostNode;
import com.wieseke.cptk.common.api.INode;
import com.wieseke.cptk.common.api.IParasiteNode;

/**
 * @author Nicolas Wieseke
 * 
 */
public abstract class Cophylogeny implements ICophylogeny {

	/**
	 * the corresponding file name
	 */
	private String filename;

	/**
	 * the root node of the host tree
	 */
	private IHostNode hostRoot;

	/**
	 * the root node of the parasite tree
	 */
	private IParasiteNode parasiteRoot;

	/**
	 * the event costs
	 */
	protected ReconstructionCosts costs;
	/**
	 * the reconstruction options
	 */
	protected Map<String, String> options;
	/**
	 * true if position values for the node were apllied
	 */
	protected boolean positionsApplied;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.ICophylogeny#getHostRoot()
	 */
	public IHostNode getHostRoot() {
		return hostRoot;
	}

	/**
	 * @param hostroot
	 *            the hostroot to set
	 */
	public void setHostRoot(IHostNode hostRoot) {
		this.hostRoot = hostRoot;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.ICophylogeny#getParasiteRoot()
	 */
	public IParasiteNode getParasiteRoot() {
		return parasiteRoot;
	}

	/**
	 * @param parasiteroot
	 *            the parasiteroot to set
	 */
	public void setParasiteRoot(IParasiteNode parasiteRoot) {
		this.parasiteRoot = parasiteRoot;
	}

	/**
	 * @return the filename
	 */
	public String getFilename() {
		return filename;
	}

	/**
	 * @param filename
	 *            the filename to set
	 */
	public void setFilename(String filename) {
		this.filename = filename;
	}

	/**
	 * @param node
	 * @return a list of all nodes
	 */
	public List<INode> getNodeList(INode node) {
		List<INode> nodeList = new ArrayList<INode>();
		nodeList.add(node);
		for (INode child : node.getChildren()) {
			nodeList.addAll(getNodeList(child));
		}
		return nodeList;
	}

	/**
	 * @return the costs
	 */
	public ReconstructionCosts getCosts() {
		return costs;
	}

	/**
	 * @param costs
	 *            the costs to set
	 */
	public void setCosts(ReconstructionCosts costs) {
		this.costs = costs;
	}

	/**
	 * @return the options
	 */
	public Map<String, String> getOptions() {
		return options;
	}

	/**
	 * @param options
	 *            the options to set
	 */
	public void setOptions(Map<String, String> options) {
		this.options = options;
	}

	/**
	 * @return the positionsApplied
	 */
	public boolean isPositionsApplied() {
		return positionsApplied;
	}

	/**
	 * @param positionsApplied
	 *            the positionsApplied to set
	 */
	public void setPositionsApplied(boolean positionsApplied) {
		this.positionsApplied = positionsApplied;
	}
}
