/**
 * 
 */
package com.wieseke.cptk.input.operation;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.wieseke.cptk.common.api.IHostNode;
import com.wieseke.cptk.input.dao.InputHostNode;
import com.wieseke.cptk.input.dao.InputParasiteNode;
import com.wieseke.cptk.input.viewer.InputCophylogenyViewer;
import com.wieseke.cptk.input.viewer.InputTreeArrangement;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EditCutHostSubtreeOperation extends AbstractOperation {

	private InputCophylogenyViewer viewer;
	private InputHostNode node;
	private int index;
	private Map<InputHostNode, List<InputParasiteNode>> oldAssociations;
	InputTreeArrangement arrangement;

	public EditCutHostSubtreeOperation(InputCophylogenyViewer viewer, InputHostNode node) {
		super("cut subtree of node " + node.getLabel());
		this.viewer = viewer;
		this.node = node;
		this.index = node.getParent().getChildren().indexOf(node);
		this.oldAssociations = new HashMap<InputHostNode, List<InputParasiteNode>>();
		fillOldAssociations(node);
		this.arrangement = new InputTreeArrangement(this.viewer.getCophylogeny());
	}

	private void fillOldAssociations(InputHostNode hostNode) {
		this.oldAssociations.put(hostNode, this.viewer.getCophylogeny().getAssociationPartners(hostNode,
				this.viewer.getCophylogeny().getParasiteRoot()));
		for (InputHostNode child : hostNode.getChildren()) {
			fillOldAssociations(child);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus execute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		if (this.node.getParent() == null)
			return Status.CANCEL_STATUS;
		return redo(monitor, info);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus redo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		IHostNode parent = (IHostNode) this.node.getParent();
		parent.getChildren().remove(this.node);
		for (Entry<InputHostNode, List<InputParasiteNode>> entry : this.oldAssociations.entrySet()) {
			for (InputParasiteNode parasiteNode : entry.getValue()) {
				parasiteNode.getAssociations().remove(entry.getKey());
			}
		}
		this.viewer.initCophylogeny(true, this.arrangement);
		this.viewer.getCophylogeny().recountAndRenumber();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus undo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		InputHostNode parent = this.node.getParent();
		parent.getChildren().add(this.index, this.node);
		for (Entry<InputHostNode, List<InputParasiteNode>> entry : this.oldAssociations.entrySet()) {
			for (InputParasiteNode parasiteNode : entry.getValue()) {
				parasiteNode.getAssociations().add(entry.getKey());
			}
		}
		this.viewer.initCophylogeny(true, this.arrangement);
		this.viewer.getCophylogeny().recountAndRenumber();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

}
