/**
 * 
 */
package com.wieseke.cptk.input.wizards;

import java.util.Random;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import com.wieseke.cptk.common.constants.ViewerConstants;
import com.wieseke.cptk.reconstruction.constants.StatisticConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public class ChooseRandomParametersWizardPage extends WizardPage {

	private Group groupRandomCycles;
	private Composite compositeRandomCycles;
	private Label labelRandomCycles;
	private Spinner spinnerRandomCycles;

	private Group groupRandomMethod;
	private Composite compositeRandomMethod;
	private Button buttonBoth;
	private Button buttonHost;
	private Button buttonParasite;
	private Button buttonAssociations;

	private Group groupTreeParameters;
	private Composite compositeTreeModel;
	private Label labelTreeModel;
	private Button buttonMarkov;
	private Button buttonAldous;
	private Button buttonPDA;
	private Button buttonAge;

	private Group groupParameters;
	private Composite compositeParameters;
	private Label labelRandomSeed;
	private Text textRandomSeed;

	public ChooseRandomParametersWizardPage() {
		super("WizardRandomNexusFileCreationPage2");
		setTitle("Random Parameters");
		setDescription("Set up the parametes for creatting the random data");
	}

	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.FILL);
		GridLayout layout = new GridLayout();
		layout.makeColumnsEqualWidth = true;
		composite.setLayout(layout);

		groupRandomCycles = new Group(composite, SWT.NONE);
		FillLayout groupRandomCyclesLayout = new FillLayout(
				org.eclipse.swt.SWT.HORIZONTAL);
		groupRandomCycles.setLayout(groupRandomCyclesLayout);
		GridData groupRandomCyclesLData = new GridData();
		groupRandomCyclesLData.horizontalAlignment = GridData.FILL;
		groupRandomCyclesLData.grabExcessHorizontalSpace = true;
		groupRandomCycles.setLayoutData(groupRandomCyclesLData);
		groupRandomCycles.setText("Random Cycles");

		compositeRandomCycles = new Composite(groupRandomCycles, SWT.NONE);
		GridLayout compositeRandomCyclesLayout = new GridLayout();
		compositeRandomCyclesLayout.makeColumnsEqualWidth = true;
		compositeRandomCycles.setLayout(compositeRandomCyclesLayout);

		labelRandomCycles = new Label(compositeRandomCycles, SWT.NONE);
		GridData labelRandomCyclesLData = new GridData();
		labelRandomCyclesLData.horizontalAlignment = GridData.FILL;
		labelRandomCyclesLData.grabExcessHorizontalSpace = true;
		labelRandomCycles.setLayoutData(labelRandomCyclesLData);
		labelRandomCycles.setText("Number of random cycles");

		spinnerRandomCycles = new Spinner(compositeRandomCycles, SWT.BORDER);
		GridData spinnerHostLeavesLData = new GridData();
		spinnerHostLeavesLData.horizontalAlignment = GridData.FILL;
		spinnerHostLeavesLData.grabExcessHorizontalSpace = true;
		spinnerRandomCycles.setLayoutData(spinnerHostLeavesLData);
		spinnerRandomCycles.setMinimum(1);
		spinnerRandomCycles.setMaximum(1000000);
		spinnerRandomCycles.setSelection(100);

		groupRandomMethod = new Group(composite, SWT.NONE);
		FillLayout groupRandomMethodLayout = new FillLayout(
				org.eclipse.swt.SWT.HORIZONTAL);
		groupRandomMethod.setLayout(groupRandomMethodLayout);
		GridData groupRandomMethodLData = new GridData();
		groupRandomMethodLData.horizontalAlignment = GridData.FILL;
		groupRandomMethodLData.grabExcessHorizontalSpace = true;
		groupRandomMethod.setLayoutData(groupRandomMethodLData);
		groupRandomMethod.setText("Random Method");

		compositeRandomMethod = new Composite(groupRandomMethod, SWT.NONE);
		GridLayout compositeRandomMethodLayout = new GridLayout();
		compositeRandomMethodLayout.makeColumnsEqualWidth = true;
		compositeRandomMethod.setLayout(compositeRandomMethodLayout);

		buttonBoth = new Button(compositeRandomMethod, SWT.RADIO | SWT.LEFT);
		buttonBoth.setText("Create random trees for host and parasite species");

		buttonHost = new Button(compositeRandomMethod, SWT.RADIO | SWT.LEFT);
		buttonHost.setText("Create random trees only for host species");

		buttonParasite = new Button(compositeRandomMethod, SWT.RADIO | SWT.LEFT);
		buttonParasite.setText("Create random trees only for parasite species");

		buttonAssociations = new Button(compositeRandomMethod, SWT.RADIO
				| SWT.LEFT);
		buttonAssociations.addSelectionListener(new SelectionListener() {

			public void widgetSelected(SelectionEvent e) {
				ChooseRandomParametersWizardPage.this.buttonMarkov
						.setEnabled(!ChooseRandomParametersWizardPage.this.buttonAssociations
								.getSelection());
				ChooseRandomParametersWizardPage.this.buttonAldous
						.setEnabled(!ChooseRandomParametersWizardPage.this.buttonAssociations
								.getSelection());
				ChooseRandomParametersWizardPage.this.buttonPDA
						.setEnabled(!ChooseRandomParametersWizardPage.this.buttonAssociations
								.getSelection());
				ChooseRandomParametersWizardPage.this.buttonAge
						.setEnabled(!ChooseRandomParametersWizardPage.this.buttonAssociations
								.getSelection());
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		buttonAssociations
				.setText("Use original trees and create random associations");
		buttonAssociations.setSelection(true);

		groupTreeParameters = new Group(composite, SWT.NONE);
		GridLayout groupTreeParametersLayout = new GridLayout();
		groupTreeParameters.setLayout(groupTreeParametersLayout);
		GridData groupTreeParametersLData = new GridData();
		groupTreeParametersLData.grabExcessHorizontalSpace = true;
		groupTreeParametersLData.horizontalAlignment = GridData.FILL;
		groupTreeParameters.setLayoutData(groupTreeParametersLData);
		groupTreeParameters.setText("Tree Parameters");

		compositeTreeModel = new Composite(groupTreeParameters, SWT.NONE);
		GridLayout compositeTreeModelLayout = new GridLayout();
		compositeTreeModelLayout.makeColumnsEqualWidth = true;
		compositeTreeModel.setLayout(compositeTreeModelLayout);
		GridData compositeTreeModelLData = new GridData();
		compositeTreeModelLData.horizontalAlignment = GridData.FILL;
		compositeTreeModelLData.grabExcessHorizontalSpace = true;
		compositeTreeModel.setLayoutData(compositeTreeModelLData);

		labelTreeModel = new Label(compositeTreeModel, SWT.NONE);
		labelTreeModel.setText("Tree Model");

		buttonMarkov = new Button(compositeTreeModel, SWT.RADIO | SWT.LEFT);
		buttonMarkov.setText("Markov (\u03b2=0)");
		buttonMarkov.setEnabled(false);

		buttonAldous = new Button(compositeTreeModel, SWT.RADIO | SWT.LEFT);
		buttonAldous.setText("Aldous (\u03b2=-1)");
		buttonAldous.setEnabled(false);

		buttonPDA = new Button(compositeTreeModel, SWT.RADIO | SWT.LEFT);
		buttonPDA.setText("PDA (\u03b2=-1.5)");
		buttonPDA.setEnabled(false);

		buttonAge = new Button(compositeTreeModel, SWT.RADIO | SWT.LEFT);
		buttonAge.setText("Age");
		buttonAge.setSelection(true);
		buttonAge.setEnabled(false);

		groupParameters = new Group(composite, SWT.NONE);
		FillLayout groupParametersLayout = new FillLayout(
				org.eclipse.swt.SWT.HORIZONTAL);
		groupParameters.setLayout(groupParametersLayout);
		GridData groupParametersLData = new GridData();
		groupParametersLData.horizontalAlignment = GridData.FILL;
		groupParametersLData.grabExcessHorizontalSpace = true;
		groupParameters.setLayoutData(groupParametersLData);
		groupParameters.setText("Parameters");

		compositeParameters = new Composite(groupParameters, SWT.NONE);
		GridLayout compositeParametersLayout = new GridLayout();
		compositeParametersLayout.makeColumnsEqualWidth = true;
		compositeParametersLayout.numColumns = 2;
		compositeParameters.setLayout(compositeParametersLayout);

		labelRandomSeed = new Label(compositeParameters, SWT.NONE);
		labelRandomSeed.setText("Random Seed Value");

		textRandomSeed = new Text(compositeParameters, SWT.BORDER);
		textRandomSeed.setText("auto");
		GridData textRandomSeedLData = new GridData();
		textRandomSeedLData.horizontalAlignment = GridData.FILL;
		textRandomSeedLData.grabExcessHorizontalSpace = true;
		textRandomSeed.setLayoutData(textRandomSeedLData);

		composite.layout();
		setControl(composite);
	}

	public int getRandomCycles() {
		if (this.spinnerRandomCycles != null) {
			return this.spinnerRandomCycles.getSelection();
		}
		return 0;
	}

	public int getRandomMethod() {
		if (this.buttonBoth.getSelection())
			return ViewerConstants.TREE_SELECTION_BOTH;
		if (this.buttonHost.getSelection())
			return ViewerConstants.TREE_SELECTION_HOST;
		if (this.buttonParasite.getSelection())
			return ViewerConstants.TREE_SELECTION_PARASITE;
		if (this.buttonAssociations.getSelection())
			return ViewerConstants.TREE_SELECTION_NONE;
		return ViewerConstants.TREE_SELECTION_NONE;
	}

	public String getModel() {
		if (this.buttonMarkov.getSelection())
			return StatisticConstants.MODEL_MARKOV;
		if (this.buttonAldous.getSelection())
			return StatisticConstants.MODEL_ALDOUS;
		if (this.buttonPDA.getSelection())
			return StatisticConstants.MODEL_PDA;
		if (this.buttonAge.getSelection())
			return StatisticConstants.MODEL_AGE;
		return StatisticConstants.MODEL_AGE;
	}

	public Random getRandom() {
		Random rand = new Random();
		{
			String seed = this.textRandomSeed.getText();
			if (seed != null && !seed.equals("")) {
				try {
					rand = new Random(Long.parseLong(seed));
				} catch (Exception e) {
				}
			}
		}
		return rand;
	}
}