/**
 * 
 */
package com.wieseke.cptk.input.wizards;

import java.awt.Dimension;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStream;
import java.util.List;

import org.apache.batik.dom.GenericDOMImplementation;
import org.apache.batik.svggen.SVGGraphics2D;
import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.dialogs.WizardExportResourcesPage;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;

import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.input.format.nexus.NexusDeserializer;
import com.wieseke.cptk.input.viewer.InputCophylogenyViewer;

/**
 * @author Nicolas Wieseke
 * 
 */
public class ExportSvgFileWizardPage extends WizardExportResourcesPage {

	/**
	 * Logger
	 */
	static Logger logger = Logger.getLogger(ExportSvgFileWizardPage.class);

	private Combo destinationNameField;
	private Button destinationBrowseButton;
	protected Button overwriteExistingFilesCheckbox;
	private static final String SELECT_DESTINATION_MESSAGE = "Select a directory to export to.";
	private static final String SELECT_DESTINATION_TITLE = "Export to Directory";

	private String DataTransfer_createTargetDirectory = "Target directory does not exist. Would you like to create it?";
	private String DataTransfer_directoryCreationError = "Target directory could not be created.";
	private String FileExport_directoryExists = "Target directory already exists as a file.";
	private String DataTransfer_browse = "B&rowse...";
	private String FileExport_toDirectory = "To director&y:";
	private String ExportFile_overwriteExisting = "&Overwrite existing files without warning";

	/**
	 * default constructor
	 * 
	 * @param pageName
	 * @param selection
	 */
	public ExportSvgFileWizardPage(String pageName,
			IStructuredSelection selection) {
		super(pageName, selection);
		setTitle("SVG File");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.dialogs.WizardExportResourcesPage#createDestinationGroup
	 * (org.eclipse.swt.widgets.Composite)
	 */
	
	protected void createDestinationGroup(Composite parent) {
		Font font = parent.getFont();
		// destination specification group
		Composite destinationSelectionGroup = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 3;
		destinationSelectionGroup.setLayout(layout);
		destinationSelectionGroup.setLayoutData(new GridData(
				GridData.HORIZONTAL_ALIGN_FILL | GridData.VERTICAL_ALIGN_FILL));
		destinationSelectionGroup.setFont(font);

		Label destinationLabel = new Label(destinationSelectionGroup, SWT.NONE);
		destinationLabel.setText(getDestinationLabel());
		destinationLabel.setFont(font);

		// destination name entry field
		destinationNameField = new Combo(destinationSelectionGroup, SWT.SINGLE
				| SWT.BORDER);
		destinationNameField.addListener(SWT.Modify, this);
		destinationNameField.addListener(SWT.Selection, this);
		GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.GRAB_HORIZONTAL);
		data.widthHint = SIZING_TEXT_FIELD_WIDTH;
		destinationNameField.setLayoutData(data);
		destinationNameField.setFont(font);

		// destination browse button
		destinationBrowseButton = new Button(destinationSelectionGroup,
				SWT.PUSH);
		destinationBrowseButton.setText(DataTransfer_browse);
		destinationBrowseButton.addListener(SWT.Selection, this);
		destinationBrowseButton.setFont(font);
		setButtonLayoutData(destinationBrowseButton);

		new Label(parent, SWT.NONE); // vertical spacer

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.dialogs.WizardDataTransferPage#createOptionsGroupButtons
	 * (org.eclipse.swt.widgets.Group)
	 */
	
	protected void createOptionsGroupButtons(Group optionsGroup) {
		Font font = optionsGroup.getFont();
		createOverwriteExisting(optionsGroup, font);
		// super.createOptionsGroupButtons(optionsGroup);
	}

	/**
	 * Create the button for checking if we should ask if we are going to
	 * overwrite existing files.
	 * 
	 * @param optionsGroup
	 * @param font
	 */
	protected void createOverwriteExisting(Group optionsGroup, Font font) {
		overwriteExistingFilesCheckbox = new Button(optionsGroup, SWT.CHECK
				| SWT.LEFT);
		overwriteExistingFilesCheckbox.setText(ExportFile_overwriteExisting);
		overwriteExistingFilesCheckbox.setFont(font);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.
	 * Event)
	 */
	
	public void handleEvent(Event event) {
		Widget source = event.widget;

		if (source == destinationBrowseButton) {
			handleDestinationBrowseButtonPressed();
		}
	}

	/**
	 * Open an appropriate destination browser so that the user can specify a
	 * source to import from
	 */
	protected void handleDestinationBrowseButtonPressed() {
		DirectoryDialog dialog = new DirectoryDialog(getContainer().getShell(),
				SWT.SAVE | SWT.SHEET);
		dialog.setMessage(SELECT_DESTINATION_MESSAGE);
		dialog.setText(SELECT_DESTINATION_TITLE);
		dialog.setFilterPath(getDestinationValue());
		String selectedDirectoryName = dialog.open();

		if (selectedDirectoryName != null) {
			setErrorMessage(null);
			setDestinationValue(selectedDirectoryName);
		}
	}

	/**
	 * Answer the string to display in self as the destination type
	 * 
	 * @return java.lang.String
	 */
	protected String getDestinationLabel() {
		return FileExport_toDirectory;
	}

	/**
	 * Answer the contents of self's destination specification widget
	 * 
	 * @return java.lang.String
	 */
	protected String getDestinationValue() {
		return destinationNameField.getText().trim();
	}

	/**
	 * Set the contents of the receivers destination specification widget to the
	 * passed value
	 * 
	 */
	protected void setDestinationValue(String value) {
		destinationNameField.setText(value);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.dialogs.WizardExportResourcesPage#getSelectedResources()
	 */
	
	public List<?> getSelectedResources() {
		return super.getSelectedResources();
	}

	/**
	 * @return true if the files should be overridden
	 */
	public boolean getOverrideValue() {
		return this.overwriteExistingFilesCheckbox.getSelection();
	}

	/**
	 * The Finish button was pressed. Try to do the required work now and answer
	 * a boolean indicating success. If false is returned then the wizard will
	 * not close.
	 * 
	 * @return boolean
	 */
	@SuppressWarnings("unchecked")
	public boolean finish() {
		if (!ensureTargetIsValid(new File(getDestinationValue()))) {
			return false;
		}

		// Save dirty editors if possible but do not stop if not all are saved
		saveDirtyEditors();
		// about to invoke the operation so save our state
		saveWidgetValues();

		List<IResource> resources = (List<IResource>) getSelectedResources();
		String destination = getDestinationValue();

		for (IResource rescource : resources) {
			try {
				IFile file = rescource.getWorkspace().getRoot().getFile(
						rescource.getFullPath());
				InputStream inputStream = file.getContents();

				NexusDeserializer nexusDeserializer = new NexusDeserializer();

				InputCophylogeny cophylogeny = nexusDeserializer.deserialize(
						inputStream, file.getName());
				InputCophylogenyViewer viewer = new InputCophylogenyViewer(
						getShell(), cophylogeny, "SWT Export");

				File path = new File(destination
						+ rescource.getFullPath().removeLastSegments(1));
				if (!path.exists()) {
					FileUtils.forceMkdir(path);
				}

				File svgFile = new File(destination + rescource.getFullPath()
						+ ".svg");
				if (!svgFile.exists()
						|| getOverrideValue()
						|| queryYesNoQuestion("File "
								+ svgFile.getName()
								+ " already exists. Would you like to override it?")) {
					svgFile.createNewFile();
					// Get a DOMImplementation.
					DOMImplementation domImpl = GenericDOMImplementation
							.getDOMImplementation();

					// Create an instance of org.w3c.dom.Document.
					Document document = domImpl.createDocument(
							"http://www.w3.org/2000/svg", "svg", null);

					// Create an instance of the SVG Generator.
					SVGGraphics2D svgGenerator = new SVGGraphics2D(document);

					// set the size
					svgGenerator.setSVGCanvasSize(new Dimension(viewer
							.getMinWidth(), viewer.getMinHeight()));

					// Ask the cophylogeny to render into the SVG Graphics2D
					// implementation.
					viewer.paintContent(svgGenerator);

					// Finally, stream out SVG to the standard output using
					FileWriter fw = new FileWriter(svgFile);
					svgGenerator.stream(fw, true);
					fw.close();
				}

			} catch (Exception e) {
				logger.warn("Could not convert file to SVG.", e);
			}
		}
		return true;
	}

	/**
	 * If the target for export does not exist then attempt to create it. Answer
	 * a boolean indicating whether the target exists (ie.- if it either
	 * pre-existed or this method was able to create it)
	 * 
	 * @return boolean
	 */
	protected boolean ensureTargetIsValid(File targetDirectory) {
		if (targetDirectory.exists() && !targetDirectory.isDirectory()) {
			displayErrorDialog(FileExport_directoryExists);
			giveFocusToDestination();
			return false;
		}

		return ensureDirectoryExists(targetDirectory);
	}

	/**
	 * Attempts to ensure that the specified directory exists on the local file
	 * system. Answers a boolean indicating success.
	 * 
	 * @return boolean
	 * @param directory
	 *            java.io.File
	 */
	protected boolean ensureDirectoryExists(File directory) {
		if (!directory.exists()) {
			if (!queryYesNoQuestion(DataTransfer_createTargetDirectory)) {
				return false;
			}

			if (!directory.mkdirs()) {
				displayErrorDialog(DataTransfer_directoryCreationError);
				giveFocusToDestination();
				return false;
			}
		}

		return true;
	}

	/**
	 * Set the current input focus to self's destination entry field
	 */
	protected void giveFocusToDestination() {
		destinationNameField.setFocus();
	}
}
