/**
 * 
 */
package com.wieseke.cptk.input.wizards;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Text;

import com.wieseke.cptk.common.constants.OptionsConstants;
import com.wieseke.cptk.common.dao.ReconstructionCosts;
import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.reconstruction.job.BaseSolverJob;

/**
 * @author Nicolas Wieseke
 * 
 */
public class RunCophylogenyReconstructionWizard extends Wizard {

	private ChooseCophylogenyCostsWizardPage chooseCophylogenyCostsWizardPage;
	private ChooseCophylogenyOptionsWizardPage chooseCophylogenyOptionsWizardPage;
	private InputCophylogeny cophylogeny;

	private ReconstructionCosts costs;
	private Map<String, String> options;

	public RunCophylogenyReconstructionWizard(InputCophylogeny cophylogeny) {
		this.cophylogeny = cophylogeny;
		setWindowTitle("Reconstruction Properties");
		setNeedsProgressMonitor(true);
	}

	
	public boolean performFinish() {
		Map<String, Text> texts = chooseCophylogenyOptionsWizardPage.getTexts();
		for (Entry<String, Text> entry : texts.entrySet()) {
			if (!OptionsConstants.checkOption(entry.getKey(), entry.getValue()
					.getText())) {
				String message = "Error parsing the option " + entry.getKey()
						+ ".";
				chooseCophylogenyCostsWizardPage.setMessage(message);
				chooseCophylogenyOptionsWizardPage.setMessage(message);
				return false;
			}
		}

		this.costs = new ReconstructionCosts();
		this.options = new HashMap<String, String>();
		if (chooseCophylogenyCostsWizardPage.useProbabilisticValues()) {
			for (Entry<String, Scale> entry : chooseCophylogenyCostsWizardPage
					.getPScales().entrySet()) {
				BigDecimal value = chooseCophylogenyCostsWizardPage
						.transformFromControlValue(entry.getValue()
								.getSelection());
				this.costs.put(entry.getKey(), value);
			}
			this.options.put(OptionsConstants.PROBABILITY_COSTS_ID,
					OptionsConstants.PROBABILITY_COSTS_1_ID);
		} else {
			for (Entry<String, Text> entry : chooseCophylogenyCostsWizardPage
					.getATexts().entrySet()) {
				BigDecimal value = BigDecimal.ZERO;
				try {
					value = new BigDecimal(entry.getValue().getText());
				} catch (NumberFormatException e) {
					String message = "\"" + entry.getValue().getText()
							+ "\" is no valid input for the "
							+ entry.getKey().toLowerCase() + "costs.";
					chooseCophylogenyCostsWizardPage.setMessage(message);
					chooseCophylogenyOptionsWizardPage.setMessage(message);
					return false;
				}
				this.costs.put(entry.getKey(), value);
			}
			this.options.put(OptionsConstants.PROBABILITY_COSTS_ID,
					OptionsConstants.PROBABILITY_COSTS_0_ID);
		}

		for (Entry<String, Combo> entry : chooseCophylogenyOptionsWizardPage
				.getCombos().entrySet()) {
			String value = String.valueOf(entry.getValue().getSelectionIndex());
			this.options.put(entry.getKey(), value);
		}
		for (Entry<String, Text> entry : chooseCophylogenyOptionsWizardPage
				.getTexts().entrySet()) {
			String value = entry.getValue().getText();
			this.options.put(entry.getKey(), value);
		}

		BaseSolverJob job = new BaseSolverJob("Compute reconstructions for "
				+ this.cophylogeny.getFilename(), this.cophylogeny, this.costs,
				this.options);
		job.setUser(true);
		job.schedule();

		return true;
	}

	
	public void addPages() {
		chooseCophylogenyCostsWizardPage = new ChooseCophylogenyCostsWizardPage(
				"Change Costs", this.cophylogeny);
		addPage(chooseCophylogenyCostsWizardPage);
		chooseCophylogenyOptionsWizardPage = new ChooseCophylogenyOptionsWizardPage(
				"Change Options", this.cophylogeny);
		addPage(chooseCophylogenyOptionsWizardPage);
		super.addPages();
	}

	/**
	 * @return the costs
	 */
	public ReconstructionCosts getCosts() {
		return costs;
	}

	/**
	 * @return the options
	 */
	public Map<String, String> getOptions() {
		return options;
	}

}
