/**
 * 
 */
package com.wieseke.cptk.main;

import java.io.FileInputStream;
import java.util.Map;
import java.util.Map.Entry;

import com.wieseke.cptk.common.constants.OptionsConstants;
import com.wieseke.cptk.common.dao.ReconstructionCosts;
import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.input.format.nexus.NexusDeserializer;
import com.wieseke.cptk.output.dao.OutputCophylogeny;
import com.wieseke.cptk.reconstruction.dao.ReconstructionCophylogeny;

/**
 * @author Nicolas Wieseke
 * 
 */
public class Bootstrapping {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length == 0 || args[0].equalsIgnoreCase("-h")
				|| args[0].equalsIgnoreCase("-help")) {
			printHelp();
			return;
		}
		try {
			String filename = args[0];

			NexusDeserializer nd = new NexusDeserializer();
			InputCophylogeny cophylogeny = nd.deserialize(new FileInputStream(
					filename), filename);

			double zeroCosts[] = { 0d, 0d, 0d, 0d };
			cophylogeny.setCosts(new ReconstructionCosts(zeroCosts));
			cophylogeny.getOptions().put(OptionsConstants.AUTOMATIC_COSTS_ID,
					OptionsConstants.AUTOMATIC_COSTS_0_ID);
			cophylogeny.getOptions().put(OptionsConstants.DUPLICATION_ID,
					OptionsConstants.DUPLICATION_0_ID);

			ReconstructionCophylogeny reconstructionCophylogeny = new ReconstructionCophylogeny(
					cophylogeny);

			reconstructionCophylogeny.compute(null);

			OutputCophylogeny outputCophylogeny = new OutputCophylogeny(
					reconstructionCophylogeny);

			BootstrappingHelper bs = new BootstrappingHelper(outputCophylogeny);

			Map<Double, Long> values = bs.computeEventsFrequencies();
			int i = 0;
			for (Entry<Double, Long> entry : values.entrySet()) {
				System.out.println(entry.getKey() + " " + entry.getValue());
				if (i > 100)
					break;
			}

			// long assocFrequencies[][] = bs.computeAssociationFrequencies();
			//
			// // System.out.println(assocFrequencies);
			// System.out.println(bs.bestReconstruction);
			//
			// Map<Double, Long> values2 = bs.rateReconstructions();
			//
			// for (Entry<Double, Long> entry : values2.entrySet()) {
			// System.out.println(entry.getKey() + " " + entry.getValue());
			// }
			// System.out.println("Best: " + bs.bestReconstruction);

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * prints the help information
	 * */
	static public void printHelp() {
		System.out
				.println("\nSyntax:\n"
						+ "java -cp .:./* com.wieseke.cptk.main.Main [filename.nex]"
						+ "\n\nFor the java -cp parameter use ';' as classpath separanor on Windows machines (not ':').");
	}
}
