/**
 * 
 */
package com.wieseke.cptk.main;

import java.io.FileInputStream;

import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.input.dao.InputNode;
import com.wieseke.cptk.input.format.nexus.NexusDeserializer;

/**
 * @author Nicolas Wieseke
 * 
 */
public class ViroBonnLCATranslation {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length == 0 || args[0].equalsIgnoreCase("-h")
				|| args[0].equalsIgnoreCase("-help")) {
			printHelp();
			return;
		}
		try {
			String filename = args[0];
			NexusDeserializer nd = new NexusDeserializer();
			InputCophylogeny cophylogeny = nd.deserialize(new FileInputStream(
					filename), filename);

			System.out.println(getTranslation(cophylogeny.getHostRoot()));
			System.out.println();
			System.out.println(getTranslation(cophylogeny.getParasiteRoot()));

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static String getTranslation(InputNode node) {
		String result = "";
		if (node.getChildren().size() > 0) {
			result += node.getLabel() + "\t" + getMostLeftLeaf(node) + "_"
					+ getMostRightLeaf(node) + "\n";
			for (InputNode child : node.getChildren()) {
				result += getTranslation(child);
			}
		}
		return result;
	}

	private static String getMostLeftLeaf(InputNode node) {
		if (node.getChildren().size() > 0) {
			return getMostLeftLeaf(node.getChildren().get(0));
		}
		return node.getLabel();
	}

	private static String getMostRightLeaf(InputNode node) {
		if (node.getChildren().size() > 0) {
			return getMostRightLeaf(node.getChildren().get(
					node.getChildren().size() - 1));
		}
		return node.getLabel();
	}

	/**
	 * prints the help information
	 * */
	static public void printHelp() {
		System.out
				.println("\nSyntax:\n"
						+ "java -cp .:./* com.wieseke.cptk.main.Reconstruction [filename.nex]"
						+ "\n\nFor the java -cp parameter use ';' as classpath separanor on Windows machines (not ':').");
	}
}
