/**
 * 
 */
package com.wieseke.cptk.output.model;

import java.math.BigDecimal;

import com.wieseke.cptk.common.dao.ReconstructionCosts;
import com.wieseke.cptk.reconstruction.dao.ReconstructionCophylogeny;
import com.wieseke.cptk.reconstruction.dao.ReconstructionSolution;

/**
 * @author Nic
 * 
 */
public class OutputReconstruction {

	private OutputReconstructions parent;
	private ReconstructionSolution solution;
	private OutputReconstructionProperties[] properties;
	private ReconstructionCophylogeny reconstructionCophylogeny;

	public OutputReconstruction(OutputReconstructions parent,
			ReconstructionSolution solution,
			ReconstructionCophylogeny reconstructionCophylogeny) {
		this.reconstructionCophylogeny = reconstructionCophylogeny;
		this.parent = parent;
		this.solution = solution;
		this.properties = new OutputReconstructionProperties[10];
		this.properties[0] = new OutputReconstructionProperties(this,
				"quality", solution.getQuality().toPlainString());
		this.properties[1] = new OutputReconstructionProperties(this,
				"total costs", solution.getTotalCosts().toPlainString());
		this.properties[2] = new OutputReconstructionProperties(this,
				"cospeciations", String.valueOf(solution.getEvents()
						.getCospeciations()));
		this.properties[3] = new OutputReconstructionProperties(this,
				"sortings", String.valueOf(solution.getEvents().getSortings()));
		this.properties[4] = new OutputReconstructionProperties(this,
				"duplications", String.valueOf(solution.getEvents()
						.getDuplications()));
		this.properties[5] = new OutputReconstructionProperties(this,
				"hostswitches", String.valueOf(solution.getEvents()
						.getHostswitches()));
		this.properties[6] = new OutputReconstructionProperties(this,
				"cospeciation cost", solution.getCosts().getCospeciationCosts()
						.toPlainString());
		this.properties[7] = new OutputReconstructionProperties(this,
				"sorting cost", solution.getCosts().getSortingCosts()
						.toPlainString());
		this.properties[8] = new OutputReconstructionProperties(this,
				"duplication cost", solution.getCosts().getDuplicationCosts()
						.toPlainString());
		this.properties[9] = new OutputReconstructionProperties(this,
				"hostswitch cost", solution.getCosts().getHostswitchCosts()
						.toPlainString());
	}

	public Object[] getChildren() {
		return properties;
	}

	public Object getParent() {
		return parent;
	}

	public BigDecimal getQuality() {
		return this.solution.getQuality();
	}

	public ReconstructionCosts getCosts() {
		return this.solution.getCosts();
	}

	public boolean hasChildren() {
		if (solution != null)
			return true;
		return false;
	}

	/**
	 * @return the solution
	 */
	public String getLabel() {
		return solution.getEvents().toString();
	}

	public ReconstructionCophylogeny getReconstructionCophylogeny() {
		return this.reconstructionCophylogeny;
	}

}
