package com.wieseke.cptk.simulation;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

public class SimulationPositionDifferences {

	private int totalNodes;
	private int exactMapping;
	private int higherMapping;
	private int lowerMapping;
	private int distinctMapping;
	private Map<Integer, Integer> mappingDifferences;

	public SimulationPositionDifferences(int totalNodes) {
		this.totalNodes = totalNodes;
		this.exactMapping = 0;
		this.higherMapping = 0;
		this.lowerMapping = 0;
		this.distinctMapping = 0;
		this.mappingDifferences = new HashMap<Integer, Integer>();
	}

	public int addExactMapping() {
		return ++this.exactMapping;
	}

	public int addHigherMapping() {
		return ++this.higherMapping;
	}

	public int addLowerMapping() {
		return ++this.lowerMapping;
	}

	public int addDistinctMapping() {
		return ++this.distinctMapping;
	}

	public int addMappingDifferences(int difference) {
		int differenceValue = this.mappingDifferences.containsKey(difference) ? this.mappingDifferences
				.get(difference)
				: 0;
		differenceValue++;
		this.mappingDifferences.put(difference, differenceValue);
		return differenceValue;
	}

	public void addSimulationPositionDifferences(
			SimulationPositionDifferences simulationDifferences) {
		this.exactMapping += simulationDifferences.getExactMapping();
		this.higherMapping += simulationDifferences.getHigherMapping();
		this.lowerMapping += simulationDifferences.getLowerMapping();
		this.distinctMapping += simulationDifferences.getDistinctMapping();
		for (Entry<Integer, Integer> entry : simulationDifferences
				.getMappingDifferences().entrySet()) {
			int mappingDifference = (this.mappingDifferences.containsKey(entry
					.getKey())) ? (int) this.mappingDifferences.get(entry
					.getKey()) : 0;
			mappingDifference += (int) entry.getValue();
			this.mappingDifferences.put(entry.getKey(), mappingDifference);
		}

	}

	/**
	 * @return the exactMapping
	 */
	public int getExactMapping() {
		return exactMapping;
	}

	/**
	 * @return the higherMapping
	 */
	public int getHigherMapping() {
		return higherMapping;
	}

	/**
	 * @return the lowerMapping
	 */
	public int getLowerMapping() {
		return lowerMapping;
	}

	/**
	 * @return the distinctMapping
	 */
	public int getDistinctMapping() {
		return distinctMapping;
	}

	/**
	 * @return the mappingDifferences
	 */
	public Map<Integer, Integer> getMappingDifferences() {
		return mappingDifferences;
	}

	/**
	 * @return the totalNodes
	 */
	public int getParasiteNodes() {
		return totalNodes;
	}

	/**
	 * @return the internalNodes
	 */
	public int getInternalNodes() {
		return exactMapping + higherMapping + lowerMapping + distinctMapping;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		int internalNodes = getInternalNodes();
		return totalNodes
				+ "\t"
				+ (double) (exactMapping + (totalNodes - internalNodes))
						/ (double) totalNodes + "\t"
				+ (double) higherMapping / (double) totalNodes + "\t"
				+ (double) lowerMapping / (double) totalNodes + "\t"
				+ (double) distinctMapping / (double) totalNodes + "\t"
				+ internalNodes + "\t"
				+ (double) exactMapping / (double) internalNodes + "\t"
				+ (double) higherMapping / (double) internalNodes + "\t"
				+ (double) lowerMapping / (double) internalNodes + "\t"
				+ (double) distinctMapping / (double) internalNodes + "\t"
				+ exactMapping + "\t" + higherMapping + "\t" + lowerMapping
				+ "\t" + distinctMapping + "\t" + mappingDifferences;
	}

}
